@@ -2,24 +2,35 @@
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
-from django.dispatch import receiver
+from django.template.defaultfilters import floatformat
 from django.test import TestCase
 from django.test.client import Client
 from rosetta.conf import settings as rosetta_settings
 from rosetta.signals import entry_changed, post_save
-import os, shutil, django
+import os
+import shutil
+import django
+
+
+try:
+    from django.dispatch import receiver
+except ImportError:
+    # We might be in django < 1.3, so backport this function
+    def receiver(signal, **kwargs):
+        def _decorator(func):
+            signal.connect(func, **kwargs)
+            return func
+        return _decorator
 
 
 class RosettaTestCase(TestCase):
     urls = 'rosetta.tests.urls'
-    
-    
-    def __init__(self, *args,**kwargs):
-        super(RosettaTestCase,self).__init__(*args,**kwargs)
+
+    def __init__(self, *args, **kwargs):
+        super(RosettaTestCase, self).__init__(*args, **kwargs)
         self.curdir = os.path.dirname(__file__)
         self.dest_file = os.path.normpath(os.path.join(self.curdir, '../locale/xx/LC_MESSAGES/django.po'))
-        self.django_version_major, self.django_version_minor = django.VERSION[0],django.VERSION[1]
-        
+        self.django_version_major, self.django_version_minor = django.VERSION[0], django.VERSION[1]
 
     def setUp(self):
         user    = User.objects.create_user('test_admin', 'test@test.com', 'test_password')
@@ -267,28 +278,37 @@ class RosettaTestCase(TestCase):
         r = self.client.get(reverse('rosetta-pick-file'))
         
         self.assertTrue('<td class="ch-messages r">1</td>' in r.content)
-        self.assertTrue('<td class="ch-progress r">0.00%</td>' in r.content)
+        self.assertTrue('<td class="ch-progress r">%s%%</td>' % str(floatformat(0.0, 2)) in r.content)
         self.assertTrue('<td class="ch-obsolete r">1</td>' in r.content)
         
         # reset the original file
         shutil.move(self.dest_file+'.orig', self.dest_file)
-    
+
     def test_11_issue_80_tab_indexes(self):
-        self.client.get(reverse('rosetta-pick-file')+'?filter=third-party')
-        r = self.client.get(reverse('rosetta-language-selection', args=('xx',0,), kwargs=dict() ))
+        self.client.get(reverse('rosetta-pick-file') + '?filter=third-party')
+        r = self.client.get(reverse('rosetta-language-selection', args=('xx', 0,), kwargs=dict()))
         r = self.client.get(reverse('rosetta-home'))
         self.assertTrue('tabindex="3"' in r.content)
 
 
     def test_12_issue_82_staff_user(self):
+        settings.ROSETTA_REQUIRES_AUTH = True
+
         self.client3 = Client()
         self.client3.login(username='test_admin3',password='test_password')
 
-        
-        r = self.client3.get(reverse('rosetta-language-selection', args=('xx',0,), kwargs=dict() ) +'?rosetta')
+        self.client3.get(reverse('rosetta-pick-file')+'?filter=third-party')
+        r = self.client3.get(reverse('rosetta-language-selection', args=('xx',0,), kwargs=dict() ))
         r = self.client3.get(reverse('rosetta-home'))
         self.assertTrue(not r.content)
 
+        settings.ROSETTA_REQUIRES_AUTH = False
+
+        self.client3.get(reverse('rosetta-pick-file')+'?filter=third-party')
+        r = self.client3.get(reverse('rosetta-language-selection', args=('xx',0,), kwargs=dict() ))
+        r = self.client3.get(reverse('rosetta-home'))
+        self.assertFalse(not r.content)
+
     
     def test_13_catalog_filters(self):
         settings.LANGUAGES = (('fr','French'),('xx','Dummy Language'),)
@@ -408,6 +428,32 @@ class RosettaTestCase(TestCase):
         self.assertTrue(self.test_16_has_request)
         del(self.test_16_has_request)
         # reset the original file
-        shutil.move(self.dest_file+'.orig', self.dest_file)
+        shutil.move(self.dest_file + '.orig', self.dest_file)
 
+    def test_17_Test_Issue_gh24(self):
+        shutil.copy(self.dest_file, self.dest_file + '.orig')
+        shutil.copy(os.path.normpath(os.path.join(self.curdir, './django.po.issue24gh.template')), self.dest_file)
 
+        self.client.get(reverse('rosetta-pick-file') + '?filter=third-party')
+        r = self.client.get(reverse('rosetta-language-selection', args=('xx', 0, ), kwargs=dict()))
+        r = self.client.get(reverse('rosetta-home'))
+
+        self.assertTrue('m_bb9d8fe6159187b9ea494c1b313d23d4' in r.content)
+
+        # post a translation, it should have properly wrapped lines
+        r = self.client.post(reverse('rosetta-home'), dict(m_bb9d8fe6159187b9ea494c1b313d23d4='Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Aenean commodo ligula eget dolor. Aenean massa. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Donec quam felis, ultricies nec, pellentesque eu, pretium quis, sem. Nulla consequat massa quis enim. Donec pede justo, fringilla vel, aliquet nec, vulputate eget, arcu. In enim justo, rhoncus ut, imperdiet a, venenatis vitae, justo. Nullam dictum felis eu pede mollis pretium.', _next='_next'))
+        pofile_content = open(self.dest_file, 'r').read()
+        self.assertTrue('"pede mollis pretium."' in pofile_content)
+
+        # Again, with unwrapped lines
+        shutil.copy(os.path.normpath(os.path.join(self.curdir, './django.po.issue24gh.template')), self.dest_file)
+        self.client.get(reverse('rosetta-pick-file') + '?filter=third-party')
+        r = self.client.get(reverse('rosetta-language-selection', args=('xx', 0, ), kwargs=dict()))
+        r = self.client.get(reverse('rosetta-home'))
+        self.assertTrue('m_bb9d8fe6159187b9ea494c1b313d23d4' in r.content)
+        rosetta_settings.POFILE_WRAP_WIDTH = 0
+        r = self.client.post(reverse('rosetta-home'), dict(m_bb9d8fe6159187b9ea494c1b313d23d4='Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Aenean commodo ligula eget dolor. Aenean massa. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Donec quam felis, ultricies nec, pellentesque eu, pretium quis, sem. Nulla consequat massa quis enim. Donec pede justo, fringilla vel, aliquet nec, vulputate eget, arcu. In enim justo, rhoncus ut, imperdiet a, venenatis vitae, justo. Nullam dictum felis eu pede mollis pretium.', _next='_next'))
+        pofile_content = open(self.dest_file, 'r').read()
+        self.assertTrue('felis eu pede mollis pretium."' in pofile_content)
+
+        shutil.move(self.dest_file + '.orig', self.dest_file)
